@@ -3,6 +3,7 @@ import json
 import uuid
 
 import dask
+from dask_cloudprovider.config import ClusterConfig
 from dask_cloudprovider.generic.vmcluster import (
     VMCluster,
     VMInterface,
@@ -12,9 +13,10 @@ from dask_cloudprovider.generic.vmcluster import (
 from dask_cloudprovider.exceptions import ConfigError
 
 try:
-    from azure.common.credentials import get_azure_cli_credentials
+    from azure.common.credentials import get_cli_profile
     from azure.mgmt.network import NetworkManagementClient
     from azure.mgmt.compute import ComputeManagementClient
+    from azure.identity import DefaultAzureCredential
 except ImportError as e:
     msg = (
         "Dask Cloud Provider Azure requirements are not installed.\n\n"
@@ -312,6 +314,8 @@ class AzureVMCluster(VMCluster):
         By default the ``daskdev/dask:latest`` image will be used.
     docker_args: string (optional)
         Extra command line arguments to pass to Docker.
+    extra_bootstrap: list[str] (optional)
+        Extra commands to be run during the bootstrap phase.
     silence_logs: bool
         Whether or not we should silence logging when setting up the cluster.
     asynchronous: bool
@@ -460,79 +464,63 @@ class AzureVMCluster(VMCluster):
         marketplace_plan: dict = {},
         **kwargs,
     ):
-        self.config = dask.config.get("cloudprovider.azure.azurevm", {})
+        self.config = ClusterConfig(dask.config.get("cloudprovider.azure", {}))
         self.scheduler_class = AzureVMScheduler
         self.worker_class = AzureVMWorker
-        self.location = (
-            location
-            if location is not None
-            else dask.config.get("cloudprovider.azure.location")
-        )
+        self.location = self.config.get("location", override_with=location)
         if self.location is None:
             raise ConfigError("You must configure a location")
-        self.resource_group = (
-            resource_group
-            if resource_group is not None
-            else self.config.get("resource_group")
+        self.resource_group = self.config.get(
+            "resource_group", override_with=resource_group
         )
         if self.resource_group is None:
             raise ConfigError("You must configure a resource_group")
-        self.public_ingress = (
-            public_ingress
-            if public_ingress is not None
-            else self.config.get("public_ingress")
+        self.public_ingress = self.config.get(
+            "azurevm.public_ingress", override_with=public_ingress
         )
-        self.credentials, self.subscription_id = get_azure_cli_credentials()
+        self.subscription_id = get_cli_profile().get_subscription_id()
+        self.credentials = DefaultAzureCredential()
         self.compute_client = ComputeManagementClient(
             self.credentials, self.subscription_id
         )
         self.network_client = NetworkManagementClient(
             self.credentials, self.subscription_id
         )
-        self.vnet = vnet if vnet is not None else self.config.get("vnet")
+        self.vnet = self.config.get("azurevm.vnet", override_with=vnet)
         if self.vnet is None:
             raise ConfigError("You must configure a vnet")
-        self.security_group = (
-            security_group
-            if security_group is not None
-            else self.config.get("security_group")
+        self.security_group = self.config.get(
+            "azurevm.security_group", override_with=security_group
         )
         if self.security_group is None:
             raise ConfigError(
                 "You must configure a security group which allows traffic on 8786 and 8787"
             )
-        self.vm_size = vm_size if vm_size is not None else self.config.get("vm_size")
-        self.disk_size = (
-            disk_size if disk_size is not None else self.config.get("disk_size")
-        )
+        self.vm_size = self.config.get("azurevm.vm_size", override_with=vm_size)
+        self.disk_size = self.config.get("azurevm.disk_size", override_with=disk_size)
         if self.disk_size > 1023:
             raise ValueError(
                 "VM OS disk canot be larger than 1023. Please change the ``disk_size`` config option."
             )
-        self.scheduler_vm_size = (
-            scheduler_vm_size
-            if scheduler_vm_size is not None
-            else self.config.get("scheduler_vm_size")
+        self.scheduler_vm_size = self.config.get(
+            "azurevm.scheduler_vm_size", override_with=scheduler_vm_size
         )
         if self.scheduler_vm_size is None:
             self.scheduler_vm_size = self.vm_size
         self.gpu_instance = (
             "_NC" in self.vm_size.upper() or "_ND" in self.vm_size.upper()
         )
-        self.vm_image = self.config.get("vm_image")
+        self.vm_image = self.config.get("azurevm.vm_image")
         for key in vm_image:
             self.vm_image[key] = vm_image[key]
-        self.bootstrap = (
-            bootstrap if bootstrap is not None else self.config.get("bootstrap")
+        self.bootstrap = self.config.get("azurevm.bootstrap", override_with=bootstrap)
+        self.auto_shutdown = self.config.get(
+            "azurevm.auto_shutdown", override_with=auto_shutdown
         )
-        self.auto_shutdown = (
-            auto_shutdown
-            if auto_shutdown is not None
-            else self.config.get("auto_shutdown")
-        )
-        self.docker_image = docker_image or self.config.get("docker_image")
         self.debug = debug
-        self.marketplace_plan = marketplace_plan or self.config.get("marketplace_plan")
+        self.marketplace_plan = marketplace_plan or self.config.get(
+            "azurevm.marketplace_plan"
+        )
         if self.marketplace_plan:
             # Check that self.marketplace_plan contains the right options with values
             if not all(
